package ctbrec.recorder;

import static java.nio.file.StandardOpenOption.*;

import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.file.Files;
import java.time.Instant;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.squareup.moshi.JsonAdapter;
import com.squareup.moshi.Moshi;

import ctbrec.Config;
import ctbrec.Model;
import ctbrec.Recording;
import ctbrec.Recording.State;
import ctbrec.io.InstantJsonAdapter;
import ctbrec.io.ModelJsonAdapter;
import ctbrec.sites.Site;

public class RecordingManager {
    private static final transient Logger LOG = LoggerFactory.getLogger(RecordingManager.class);

    private Config config;
    private Moshi moshi;
    private JsonAdapter<Recording> adapter;
    private List<Recording> recordings = new ArrayList<>();
    // private RecordingFileMonitor monitor = new RecordingFileMonitor(this);

    public RecordingManager(Config config, List<Site> sites) throws IOException {
        this.config = config;
        moshi = new Moshi.Builder()
                .add(Model.class, new ModelJsonAdapter(sites))
                .add(Instant.class, new InstantJsonAdapter())
                .build();
        adapter = moshi.adapter(Recording.class).indent("  ");

        loadRecordings();
        // startMonitoring();
    }

    public void add(Recording rec) throws UnsupportedEncodingException, IOException {
        saveRecording(rec);
        recordings.add(rec);
        // registerFileWatch(rec);
    }

    public void saveRecording(Recording rec) throws UnsupportedEncodingException, IOException {
        String json = adapter.toJson(rec);
        File recordingsMetaDir = getDir();
        Files.createDirectories(recordingsMetaDir.toPath());
        String filename = rec.toString() + ".json";
        File recordingMetaData = new File(recordingsMetaDir, filename);
        rec.setMetaDataFile(recordingMetaData.getAbsolutePath());
        Files.write(recordingMetaData.toPath(), json.getBytes("utf-8"), CREATE, WRITE, TRUNCATE_EXISTING);
    }

    private void loadRecordings() throws IOException {
        File recordingsMetaDir = getDir();
        File[] metaFiles = recordingsMetaDir.listFiles((file, name) -> name.endsWith(".json"));
        if (metaFiles != null) {
            for (File file : metaFiles) {
                String json = new String(Files.readAllBytes(file.toPath()), "utf-8");
                Recording recording = adapter.fromJson(json);
                if (recording.getStatus() == State.RECORDING) {
                    recording.setStatus(State.WAITING);
                }
                if (recordingExists(recording)) {
                    recordings.add(recording);
                } else {
                    LOG.info("Recording {} does not exist anymore -> deleting meta data", recording);
                    Files.deleteIfExists(new File(recording.getMetaDataFile()).toPath());
                }
            }
        }
    }

    // private void startMonitoring() {
    // for (Recording recording : recordings) {
    // registerFileWatch(recording);
    // }
    // Thread watcher = new Thread(() -> monitor.processEvents());
    // watcher.setDaemon(true);
    // watcher.setPriority(Thread.MIN_PRIORITY);
    // watcher.setName("RecordingFileMonitor");
    // watcher.start();
    // }
    //
    // private void registerFileWatch(Recording recording) {
    // File rec = recording.getAbsoluteFile();
    // if (rec.isDirectory()) {
    // monitor.register(rec.toPath());
    // } else {
    // monitor.register(rec.getParentFile().toPath());
    // }
    // }
    //
    // private void removeFileWatch(Recording recording) {
    // File rec = recording.getAbsoluteFile();
    // if (rec.isDirectory()) {
    // monitor.unregister(rec.toPath());
    // } else {
    // monitor.unregister(rec.getParentFile().toPath());
    // }
    // }

    private boolean recordingExists(Recording recording) {
        File rec = new File(config.getSettings().recordingsDir, recording.getPath());
        return rec.exists();
    }

    private File getDir() {
        File configDir = config.getConfigDir();
        File recordingsMetaDir = new File(configDir, "recordings");
        return recordingsMetaDir;
    }

    public void delete(Recording recording) throws IOException {
        int idx = recordings.indexOf(recording);
        recording = recordings.get(idx);

        recording.setStatus(State.DELETING);
        File recordingsDir = new File(config.getSettings().recordingsDir);
        File path = new File(recordingsDir, recording.getPath());
        LOG.debug("Deleting {}", path);

        // delete the video files
        if (path.isFile()) {
            Files.delete(path.toPath());
            deleteEmptyParents(path.getParentFile());
        } else {
            deleteDirectory(path);
            deleteEmptyParents(path);
        }

        // delete the meta data
        Files.deleteIfExists(new File(recording.getMetaDataFile()).toPath());

        // remove from data structure
        recordings.remove(recording);
        recording.setStatus(State.DELETED);

        // removeFileWatch(recording);
    }

    public List<Recording> getAll() {
        for (Recording recording : recordings) {
            recording.refresh();
        }
        return new ArrayList<>(recordings);
    }

    private void deleteEmptyParents(File parent) throws IOException {
        File recDir = new File(Config.getInstance().getSettings().recordingsDir);
        while (parent != null && parent.list() != null && parent.list().length == 0) {
            if (parent.equals(recDir)) {
                return;
            }
            LOG.debug("Deleting empty directory {}", parent.getAbsolutePath());
            Files.delete(parent.toPath());
            parent = parent.getParentFile();
        }
    }

    private void deleteDirectory(File directory) throws IOException {
        if (!directory.exists()) {
            return;
        }

        File[] files = directory.listFiles();
        boolean deletedAllFiles = true;
        for (File file : files) {
            try {
                LOG.trace("Deleting {}", file.getAbsolutePath());
                Files.delete(file.toPath());
            } catch (Exception e) {
                deletedAllFiles = false;
                LOG.debug("Couldn't delete {}", file, e);
            }
        }

        if (!deletedAllFiles) {
            throw new IOException("Couldn't delete all files in " + directory);
        }
    }
}
